@@ -213,8 +213,7 @@ sub check_policy ()
                                ($params{protocol_state} ne "RCPT"));
 
   ### the primary key
-  $ip = lc($params{client_address});
-  substr($ip, rindex($ip, ".") + 1) = "0";
+  $ip = &normalize_ip(lc($params{client_address}));
   $from = lc($params{sender});
   $to = lc($params{recipient});
 
@@ -259,6 +258,30 @@ sub check_policy ()
   return &choose_action($ok);
 }
 
+sub normalize_ip ()
+{
+  my $ip = shift;
+
+  if ($ip =~ /:/) {
+    # normalize ipv6 address to /64
+    my @parts = split(/::/, $ip);
+    my @left = ($#parts > -1 ? split(/:/, $parts[0]) : ());
+    my @right = ($#parts > 0 ? split(/:/, $parts[1]) : ());
+
+    my $missing = 8 - ($#left + 1) - ($#right + 1);
+    for (my $i = 0; $i < $missing; $i++) {
+      push @left, "0";
+    }
+
+    $ip = join(":", (@left, @right)[0..3]) . "::";
+  } else {
+    # normalize ipv4 address to /24
+    substr($ip, rindex($ip, ".") + 1) = "0";
+  }
+
+  return $ip;
+}
+
 sub choose_action ()
 {
   return "action=dunno\n\n" if $_[0];
